/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup I2C
  * @{
  */

/**
  * @addtogroup I2C_SlaveDMA
  * @{
  */

/* Private typedef --------------------------------------------------------------*/
I2C_InitTypeStruct I2C_InitStruct;
DMA_InitTypeStruct DMA_InitStruct;

/* Private define ---------------------------------------------------------------*/
#define NUM 5

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/
uint8_t tx_buf[NUM] = {0x11, 0x22, 0x44, 0x88, 0xAA};
uint8_t rx_buf[NUM] ;

/* Private functions ------------------------------------------------------------*/
void I2C_Slave_Transmit(uint32_t timeout);
void I2C_Slave_Receive(uint32_t timeout);

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();

  /* LED Init */
  EVAL_LedInit(LED1);

  /* Print Uart Init */
  EVAL_ComInit();

  /********************* Operation  *********************/
  /* Enable clock */
  RCC_Unlock();
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_GPIO, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_I2C, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_DMA, ENABLE);
  RCC_Lock();
  /* Cofig I2C GPIO */
  GPIO_InitTypeStruct GPIO_InitStruct;
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  GPIO_InitStruct.GPIO_OType = GPIO_OTYPE_OD;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_NOPULL;
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_4 | GPIO_PIN_5;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOD, GPIO_PIN_4 | GPIO_PIN_5, GPIO_ALT_FUNC_1);
  /* Config I2C */
  I2C_DeInit();
  I2C_StructInit(&I2C_InitStruct);
  I2C_InitStruct.I2C_OwnAddress = 0x00;
  I2C_InitStruct.I2C_Prescaler = 0x16;
  I2C_InitStruct.I2C_Timing = 0x00420F13;
  I2C_InitStruct.I2C_DigitalFilter = 0x0F;
  I2C_InitStruct.I2C_Mode = I2C_MODE_SLAVE;
  I2C_InitStruct.I2C_AddressingMode = I2C_ADDRESSING_MODE_7BIT;
  I2C_Init(&I2C_InitStruct);
  /* Enable I2C */
  I2C_Cmd(ENABLE);
  /* Enable I2C-DMA */
  I2C_DMACmd(I2C_DMA_REQ_TX, ENABLE);
  I2C_DMACmd(I2C_DMA_REQ_RX, ENABLE);
  /* I2C transmit */
  I2C_Slave_Transmit(1000);
  printf("********************* I2C Transmit  *********************\n");

  for(uint16_t i = 0; i < NUM; i++)
  {
    printf("tx_buf[%d] = 0X%x\n", i, tx_buf[i]);
  }

  /* I2C receive */
  I2C_Slave_Receive(1000);
  printf("********************* I2C Receive  *********************\n");

  for(uint16_t i = 0; i < NUM; i++)
  {
    printf("rx_buf[%d] = 0X%x\n", i, rx_buf[i]);
  }

  uint32_t ledTime = 0;
  /* Infinite loop */
  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/**
  * @brief  This function is I2C Slave Transmit.
  * @param  None
  * @retval None
  */
void I2C_Slave_Transmit(uint32_t timeout)
{
  uint32_t time = GetTick() + timeout;
  /* Clear I2C transmit data buffer register */
  I2C_ClearTxBuffer();

  /* Wait slave mode bus address match detection flag */
  while(I2C_GetIntFlagStatus(I2C_INT_FLAG_ADDRM) != SET && (time > GetTick()));

  /* Clear slave mode bus address match detection flag */
  I2C_ClearIntFlag(I2C_INT_FLAG_ADDRM);

  /* Send data */
  if(I2C_GetTransferDirection() == I2C_DIRECTION_RECEIVER)
  {
    /* Config DMA trasmit */
    DMA_InitStruct.DMA_CICRState = DISABLE;
    DMA_InitStruct.DMA_DataNum = 1 - 1;
    DMA_InitStruct.DMA_DataSize = DMA_DATA_SIZE_8;
    DMA_InitStruct.DMA_DstAddr = (uint32_t)&I2C->TDR;
    DMA_InitStruct.DMA_DstIncMode = DMA_ADDR_MODE_RESET;
    DMA_InitStruct.DMA_ErrIntState = DISABLE;
    DMA_InitStruct.DMA_FISIntState = DISABLE;
    DMA_InitStruct.DMA_ReqSource = DMA_I2C_TX_REQ;
    DMA_InitStruct.DMA_RoundNum = 5 - 1;
    DMA_InitStruct.DMA_SrcAddr = (uint32_t)tx_buf;
    DMA_InitStruct.DMA_SrcIncMode = DMA_ADDR_MODE_INCREASE;
    DMA_Init(DMA_CH0, &DMA_InitStruct);
    /* Enable DMA_CH0  */
    DMA_Cmd(DMA_CH0, ENABLE);

    while(DMA_GetIntFlagStatus(DMA_CH0, DMA_INT_FLAG_FINISH) != SET && (time > GetTick()));
  }

  /* Waiting for the STOP signal end flag */
  while(I2C_GetIntFlagStatus(I2C_INT_FLAG_STOPF) != SET && (time > GetTick()));

  /* Clear STOP flag */
  I2C_ClearIntFlag(I2C_INT_FLAG_STOPF);
}

/**
  * @brief  This function is I2C Slave Receive.
  * @param  None
  * @retval None
  */
void I2C_Slave_Receive(uint32_t timeout)
{
  uint32_t time = GetTick() + timeout;
  /* Wait slave mode bus address match detection flag */
  while(I2C_GetIntFlagStatus(I2C_INT_FLAG_ADDRM) != SET && (time > GetTick()));

  /* Clear slave mode bus address match detection flag */
  I2C_ClearIntFlag(I2C_INT_FLAG_ADDRM);

  /* Receive data */
  if(I2C_GetTransferDirection() == I2C_DIRECTION_TRANSMITTER)
  {
    /* Config DMA receive */
    DMA_InitStruct.DMA_CICRState = DISABLE;
    DMA_InitStruct.DMA_DataNum = 1 - 1;
    DMA_InitStruct.DMA_DataSize = DMA_DATA_SIZE_8;
    DMA_InitStruct.DMA_DstAddr = (uint32_t)rx_buf;
    DMA_InitStruct.DMA_DstIncMode = DMA_ADDR_MODE_INCREASE;
    DMA_InitStruct.DMA_ErrIntState = DISABLE;
    DMA_InitStruct.DMA_FISIntState = DISABLE;
    DMA_InitStruct.DMA_ReqSource = DMA_I2C_RX_REQ;
    DMA_InitStruct.DMA_RoundNum = 5 - 1;
    DMA_InitStruct.DMA_SrcAddr = (uint32_t)&I2C->RDR;
    DMA_InitStruct.DMA_SrcIncMode = DMA_ADDR_MODE_RESET;
    DMA_Init(DMA_CH1, &DMA_InitStruct);
    /* Enable DMA_CH1 */
    DMA_Cmd(DMA_CH1, ENABLE);

    while(DMA_GetIntFlagStatus(DMA_CH1, DMA_INT_FLAG_FINISH) != SET && (time > GetTick()));
  }

  /* Waiting for the STOP signal end flag */
  while(I2C_GetIntFlagStatus(I2C_INT_FLAG_STOPF) != SET && (time > GetTick()));

  /* Clear STOP flag */
  I2C_ClearIntFlag(I2C_INT_FLAG_STOPF);
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}


/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
